# Example message:
# curl -H "Content-Type: application/json" -d '{"time": "2021-07-20T05:05:47", "service": "foo", "log": "bar"}' -X POST 'https://us-central1-hip-wharf-319304.cloudfunctions.net/cf-to-pubsub'

variable "project" {
  description = "Project ID."
  type        = string
}

variable "region" {
  description = "Region."
  type        = string
}

variable "source_bucket" {
  description = "Google storage bucket where the source code will be stored."
}

variable "function_source_name" {
  description = "Name of the folder containing the source code for the function."
  type        = string
  default     = "cf_to_pubsub"
}

variable "service_cloudbuild" {
  description = "THe cloudbuild google_project_service."
}

output "https_trigger_url" {
  description = "https endpoint for the cloud function."
  value       = google_cloudfunctions_function.function.https_trigger_url
}

resource "random_id" "cf_bucket_id" {
  byte_length = 4
}

data "archive_file" "source_archive" {
  type        = "zip"
  source_dir  = "${path.module}/functions/${var.function_source_name}"
  output_path = "${path.module}/built/${var.function_source_name}.zip"
  excludes    = [".python-version"]
}

resource "google_storage_bucket_object" "remote_archive" {
  name   = "${var.function_source_name}-${data.archive_file.source_archive.output_base64sha256}.zip"
  bucket = var.source_bucket.name
  source = data.archive_file.source_archive.output_path
}

resource "google_cloudfunctions_function" "function" {
  name        = "cf-to-pubsub"
  description = "CloudFunction to PubSub"
  runtime     = "python39"

  available_memory_mb   = 128
  source_archive_bucket = var.source_bucket.name
  source_archive_object = google_storage_bucket_object.remote_archive.name
  trigger_http          = true
  entry_point           = "main"
  max_instances         = 4
  ingress_settings      = "ALLOW_ALL"
  # ingress_settings      = "ALLOW_INTERNAL_ONLY"

  depends_on = [
    var.service_cloudbuild
  ]
}

# Allow unauthenticated access over http
resource "google_cloudfunctions_function_iam_member" "invoker" {
  project        = google_cloudfunctions_function.function.project
  region         = google_cloudfunctions_function.function.region
  cloud_function = google_cloudfunctions_function.function.name

  role   = "roles/cloudfunctions.invoker"
  member = "allUsers"
}
